package dev.shadowsoffire.apotheosis.adventure.loot;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.mojang.datafixers.kinds.App;
import com.mojang.datafixers.kinds.Applicative;
import com.mojang.datafixers.util.Function6;
import com.mojang.serialization.Codec;
import com.mojang.serialization.codecs.ListCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityClamp;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.placebo.codec.CodecProvider;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import dev.shadowsoffire.placebo.reload.WeightedDynamicRegistry;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import javax.annotation.Nullable;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.Style;
import net.minecraft.network.chat.TextColor;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.util.ExtraCodecs;
import net.minecraft.util.RandomSource;
import net.minecraft.world.item.Item;
import net.minecraftforge.registries.ForgeRegistries;

public class LootRarity implements CodecProvider<LootRarity>, WeightedDynamicRegistry.ILuckyWeighted, Comparable<LootRarity> {
  public static final Codec<LootRarity> LOAD_CODEC;
  
  static {
    LOAD_CODEC = RecordCodecBuilder.create(inst -> inst.group((App)TextColor.f_237295_.fieldOf("color").forGetter(LootRarity::getColor), (App)ForgeRegistries.ITEMS.getCodec().fieldOf("material").forGetter(LootRarity::getMaterial), (App)Codec.INT.fieldOf("ordinal").forGetter(LootRarity::ordinal), (App)Codec.intRange(0, 2147483647).fieldOf("weight").forGetter(WeightedDynamicRegistry.ILuckyWeighted::getWeight), (App)PlaceboCodecs.nullableField(Codec.floatRange(0.0F, Float.MAX_VALUE), "quality", Float.valueOf(0.0F)).forGetter(WeightedDynamicRegistry.ILuckyWeighted::getQuality), (App)(new ListCodec(LootRule.CODEC)).fieldOf("rules").forGetter(LootRarity::getRules)).apply((Applicative)inst, LootRarity::new));
  }
  
  @Deprecated
  public static final Codec<DynamicHolder<LootRarity>> HOLDER_CODEC = ExtraCodecs.m_184415_(() -> {
        Objects.requireNonNull(RarityRegistry.INSTANCE);
        return Codec.STRING.xmap(RarityRegistry::convertId, ResourceLocation::toString).xmap(RarityRegistry.INSTANCE::holder, DynamicHolder::getId);
      });
  
  public static final Codec<LootRarity> CODEC = ExtraCodecs.m_184415_(() -> {
        Objects.requireNonNull(RarityRegistry.INSTANCE);
        return HOLDER_CODEC.xmap(DynamicHolder::get, RarityRegistry.INSTANCE::holder);
      });
  
  private final Item material;
  
  private final TextColor color;
  
  private final int ordinal;
  
  private final int weight;
  
  private final float quality;
  
  private final List<LootRule> rules;
  
  private LootRarity(TextColor color, Item material, int ordinal, int weight, float quality, List<LootRule> rules) {
    this.color = color;
    this.material = material;
    this.ordinal = ordinal;
    this.weight = weight;
    this.quality = quality;
    this.rules = rules;
  }
  
  public Item getMaterial() {
    return this.material;
  }
  
  public TextColor getColor() {
    return this.color;
  }
  
  public int ordinal() {
    return this.ordinal;
  }
  
  public int getWeight() {
    return this.weight;
  }
  
  public float getQuality() {
    return this.quality;
  }
  
  public List<LootRule> getRules() {
    return this.rules;
  }
  
  public LootRarity next() {
    return (LootRarity)RarityRegistry.next(RarityRegistry.INSTANCE.holder(this)).get();
  }
  
  public LootRarity prev() {
    return (LootRarity)RarityRegistry.prev(RarityRegistry.INSTANCE.holder(this)).get();
  }
  
  public boolean isAtMost(LootRarity other) {
    return (ordinal() <= other.ordinal());
  }
  
  public boolean isAtLeast(LootRarity other) {
    return (ordinal() >= other.ordinal());
  }
  
  public static LootRarity min(LootRarity a, @Nullable LootRarity b) {
    if (b == null)
      return a; 
    return (a.ordinal <= b.ordinal) ? a : b;
  }
  
  public static LootRarity max(LootRarity a, @Nullable LootRarity b) {
    if (b == null)
      return a; 
    return (a.ordinal >= b.ordinal) ? a : b;
  }
  
  public LootRarity clamp(@Nullable LootRarity lowerBound, @Nullable LootRarity upperBound) {
    return max(min(this, upperBound), lowerBound);
  }
  
  public Component toComponent() {
    return (Component)Component.m_237115_("rarity." + RarityRegistry.INSTANCE.getKey(this)).m_130948_(Style.f_131099_.m_131148_(this.color));
  }
  
  public String toString() {
    return "LootRarity{" + RarityRegistry.INSTANCE.getKey(this) + "}";
  }
  
  public int compareTo(LootRarity o) {
    return Integer.compare(this.ordinal, o.ordinal);
  }
  
  public Codec<? extends LootRarity> getCodec() {
    return LOAD_CODEC;
  }
  
  public static LootRarity random(RandomSource rand, float luck) {
    return (LootRarity)RarityRegistry.INSTANCE.getRandomItem(rand, luck);
  }
  
  public static LootRarity random(RandomSource rand, float luck, @Nullable RarityClamp clamp) {
    LootRarity rarity = random(rand, luck);
    return (clamp == null) ? rarity : clamp.clamp(rarity);
  }
  
  public static <T> Codec<Map<LootRarity, T>> mapCodec(Codec<T> codec) {
    return (Codec<Map<LootRarity, T>>)Codec.unboundedMap(CODEC, codec);
  }
}
